#!/usr/bin/rexx
/*
   Name:    setupFileAssociations.rex
   Purpose: creates files to set up and remove file associations
   Author:  Rony G. Flatscher
   Date:    2010-07-26/27
   Version: 0.01
   Changes:
            2010-07-26, ---rgf, plain


   last svn check-in: $Revision: 640 $ $Author: rony $ $Date: 2010-04-17 22:25:38 +0200 (Sat, 17 Apr 2010) $

   Usage:   rexx setupFileAssocitions.rex

   license:

    ------------------------ Apache Version 2.0 license -------------------------
       Copyright (C) 2010 Rony G. Flatscher

       Licensed under the Apache License, Version 2.0 (the "License");
       you may not use this file except in compliance with the License.
       You may obtain a copy of the License at

           http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing, software
       distributed under the License is distributed on an "AS IS" BASIS,
       WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
       See the License for the specific language governing permissions and
       limitations under the License.
    -----------------------------------------------------------------------------
*/


parse source opsys +1

dir=.directory~new   -- will contain setup relevant information
dir~runtime=.dateTime~new~string
dir~rexx_home = value("REXX_HOME", , "ENVIRONMENT")   -- fetch value of environment variable "REXX_HOME"

parse version "REXX-" rexxProcessor "_" version "("
dir~rexx.processor = rexxProcessor
dir~rexx.version   = version

if opsys='W' then    -- Windows
do
   dir~opsys="W"              -- operating system: Windows
   file.sep="\"               -- file separator
   dir~rexx_bin = "rexx.exe"                          -- name of executable
   dir~path.sep       = ";"   -- path separator
end
else                 -- Unix based
do
   dir~opsys="U"              -- operating system: Unix-like
   file.sep="/"               -- file separator
   dir~rexx_bin = "rexx"                              -- name of executable
   dir~path.sep       = ":"   -- path separator
end
dir~file.sep       = file.sep    -- save file separator as well

dir~full_path_to_rexx=dir~rexx_home || file.sep || dir~rexx_bin -- fully qualified path

   /* determine location of "images" */
dir~path_to_images="images"
if directory()~caselessPos("BSF4ooRexx.dev")>0 then -- o.k. running in the development environment
   dir~path_to_images=".."file.sep"images"

   /* define (base) names of image files */
dir~rxj.imageName="BSF4ooRexx"
dir~rex.imageName="ooRexx"
dir~rxo.imageName="ooRexx4OOo"

   /* template files (containing template code, if new file gets created via Explorer) */
dir~rex.template ="template.rex"
dir~rxj.template ="template.rxj"
dir~rxo.template ="template.rxo"

   /* define available pixel sizes of the PNG versions (for Unix) */
dir~png.imageSizes=.array~of(16,32,48,64,128,256)


dir ~ rex.Name = "RexxScript"       -- already in use on Windows for ooRexx ! (do not change!)
dir ~ rxj.Name = "RexxJavaScript"
dir ~ rxo.Name = "RexxOOoScript"

dir ~ rex.mimetype = "application/x-rexx"
dir ~ rxj.mimetype = "application/x-rexx-java"
dir ~ rxo.mimetype = "application/x-rexx-java-ooo"

   -- on Windows ".rex" is already set up
dir ~ rex.extensions = .array~of( ".cls",   ".orx",  ".rex", ".rexx", ".testUnit" )

if opsys='U' then -- add '.rex' filetype for Unix
   dir ~ rex.extensions ~ put(".rex")

dir ~ rxj.extensions = .array~of( ".jrexx", ".rexxj", ".rxj" )
dir ~ rxo.extensions = .array~of( ".rxo" )

info = "executed by" dir~rexx.processor -- "version" dir~rexx.version
dir ~rex.comment = dir~rex.name info
dir ~rxj.comment = dir~rxj.name "(Rexx script using Java)" info
dir ~rxo.comment = dir~rxo.name "(Rexx script using OpenOffice.org)" info


call dump2 dir

-- exit
if opsys='W' then call create4windows dir
             else call create4linux dir

::requires "rgf_util2.rex"


/* ================================================================================= */
/* -------------------- Windows part -------------------- */
/*
   Maybe useful URLs (as of 2010-07-27):

   - Regedit Command Line Options Syntax
      <http://techsupt.winbatch.com/ts/T000001029F18.html>
   - Regedit command-line switches
      <http://www.eolsoft.com/freeware/registry_jumper/regedit_switches.htm>
   - WindowsScript Host ("WScript.Shell")
      <http://msdn.microsoft.com/en-us/library/98591fh7%28v=VS.85%29.aspx>
   - Windows XP File Association Fixes
      <http://www.dougknox.com/xp/file_assoc.htm>
   - Customizing the Windows XP New Menu
      <http://windows-xp-dox.net/MS.Press-Insider.Power.Techniq/6843final/LiB0043.html>
   - How to add, modify, or delete registry subkeys and values by using a registration entries (.reg) file
      <http://support.microsoft.com/kb/310516>

*/
::routine create4windows
  use arg dir

  wsh  = .OleObject~new("WScript.Shell")
  hkcr = "HKEY_CLASSES_ROOT"
  file.sep = dir~file.sep
  fragmentName = "fileAssociations_for_"

  call updateRexxAssociation dir, hkcr, wsh, fragmentName
  call createNewAssociations dir, hkcr, wsh, fragmentName


  exit

   ----- change the .REX definition ------------------------------------------------
updateRexxAssociation: procedure
  use arg dir, hkcr, wsh, fragmentName

  regFile="rex"
  uRegFile=regFile~upper

   -- write prolog
  ifs = .stream~new(fragmentName || uRegfile"_add.reg")~~open("replace")
  ufs = .stream~new(fragmentName || uRegfile"_remove.reg")~~open("replace")

  ifs~~say("Windows Registry Editor Version 5.00")~~say
  ufs~~say("Windows Registry Editor Version 5.00")~~say

  ifs~~say("; Created by ""setupFileAssociations.rex"" (from BSF4ooRexx) on:" dir~runtime)~~say
  ufs~~say("; Created by ""setupFileAssociations.rex"" (from BSF4ooRexx) on:" dir~runtime)~~say

   -- add file-extension entries: do not change "HKCR\.REX" definitions
  do extension over dir~rex.extensions
     if extension='.rex' then    -- this extension should exist already
     do
        key=hkcr || "\" || extension~upper || "\"
        keyVal=queryWinRegistry(wsh, key)
        if keyVal<>.nil then           -- if set up, don't change it
        do
           ifs~~say("; skipping existing key:" pp(hkcr || "\" || extension~upper)) ~~say   -- add/update key
           iterate
        end
     end

     ifs~say(pp(hkcr || "\" || extension~upper))    -- add/update key
     ifs~say('@="REXXScript"')
     ifs~say('"Content Type"=' || enQQ(dir~rex.mimetype) )
     ifs~say('"PerceivedType"="text"')

     ufs~say(pp('-' || hkcr || "\" || extension~upper)) -- remove entire key

     ifs~say
     ufs~say
  end

   -- be nice and also add the new definitions (researched for this program) to "HKCR/.REX", if not present
      -- set mimetype information with filetype
  contentTypeKey = hkcr || "\.REX\Content Type"
  contentTypeVal = queryWinRegistry(wsh, contentTypeKey)
  if contentTypeVal=.nil then
  do
      -- add mimetype information
     ifs~say(pp(hkcr || "\.REX"))    -- add/update key
     ifs~say('"Content Type"=' || enQQ(dir~rex.mimetype) )

     ufs~say(pp(hkcr || "\.REX"))    -- remove value
     ufs~say('"Content Type"=-')
     ufs~say
  end
  else
  do
     ifs~say(";" pp(hkcr || "\.REX"))    -- add/update key
     ifs~say("; mimetype entry exists already:" pp(contentTypeKey)"="pp(contentTypeVal) )    -- add/update key
  end
  ifs~say

      -- set PerceivedType information with filetype
  perceivedTypeKey = hkcr || "\.REX\PerceivedType"
  perceivedTypeVal = queryWinRegistry(wsh, perceivedTypeKey)
  if perceivedTypeVal=.nil then
  do
      -- add mimetype information
     ifs~say(pp(hkcr || "\.REX"))    -- add/update key
     ifs~say('"PerceivedType"="text"')

     ufs~say(pp(hkcr || "\.REX"))
     ufs~say('"PerceivedType"=-')    -- remove value
     ufs~say
  end
  else
  do
     ifs~say(";" pp(hkcr || "\.REX"))    -- add/update key
     ifs~say("; PerceivedType entry exists already:" pp(perceivedTypeKey)"="pp(perceivedTypeVal) )    -- add/update key
  end
  ifs~say

      -- set template file name for ShellNew
  fname=stream("..\template\"dir~rex.template, "C", "query exists") /* from current directory */
  if fname<>"" then
  do
     shellNewKey = hkcr || "\.REX\ShellNew\"
     shellNewVal = queryWinRegistry(wsh, shellNewKey)
     if shellNewVal=.nil then
     do
         -- add mimetype information
        ifs~say(pp(hkcr || "\.REX\ShellNew"))   -- add/update key
        ifs~say('"FileName"=' || quote4reg(fname) )

        ufs~say(pp("-" ||  hkcr || "\.REX\ShellNew")) -- remove this particular key
        ufs~say
     end
     else
     do
        ifs~say(";" pp(hkcr || "\.REX\ShellNew"))    -- add/update key
        ifs~say("; key exists already:" pp(shellNewKey) )    -- add/update key
     end
     ifs~say
  end

  -- ufs~say(pp('-' || hkcr || "\" || ".REX") -- remove key ?


   -- add file-extension to MIME DataBase
  mimeTypeKey = hkcr || "\MIME\DataBase\Content Type\" || dir~rex.mimetype
  mimeTypeVal = queryWinRegistry(wsh, mimeTypeKey)
  if mimeTypeVal=.nil then
  do
     ifs~say(pp(hkcr || "\MIME\DataBase\Content Type\" || dir~rex.mimetype))  -- add/update key
     ifs~say('"Extension"=' || enQQ(".rex") )

     ufs~say(pp('-' || hkcr || "\MIME\DataBase\Content Type\" || dir~rex.mimetype))  -- remove key
     ufs~say
  end
  else
  do
     ifs~say(";" pp(mimeTypeKey) "exists already, value:" pp(mimeTypeVal))
  end
  ifs~say


   -- change Rexx icon
  iconKey = "HKEY_CLASSES_ROOT\REXXScript\DefaultIcon"
  iconVal = queryWinRegistry(wsh, iconKey"\")   -- try to get current default value

  ifs~say(pp(iconKey))     -- udpate key
  fname=stream(dir~path_to_images || "\" || dir~rex.ImageName || ".ico", "C", "query exists")
  ifs~say('@='quote4reg(fname",0"))

  ufs~say(pp(iconKey))     -- udpate key
  if iconVal=.nil then     -- no DefaultIcon key as of yet!
  do
     fname=stream(dir~rexx_home || "\" || "rexx.ico", "C", "query exists")
     ufs~say('@='quote4reg(fname",0"))
  end
  else   -- use existing value for restoring
  do
     ufs~say('@='quote4reg(iconVal))      -- restore value that was found
  end

   --- done
  ifs~close
  ufs~close

  return


   ----- add the .RXJ and .RXO definitions ------------------------------------------------
createNewAssociations: procedure
  use arg dir, hkcr, wsh, fragmentName

  -- create separate files for each file association type
  do regfile over .list~of("rxj", "rxo")
     uRegFile=regfile~upper
       -- open/create files
     ifs = .stream~new(fragmentName || uRegfile"_add.reg")~~open("replace")
     ufs = .stream~new(fragmentName || uRegfile"_remove.reg")~~open("replace")

         -- write prolog
     ifs~~say("Windows Registry Editor Version 5.00")~~say
     ufs~~say("Windows Registry Editor Version 5.00")~~say

     ifs~~say("; Created by ""setupFileAssociations.rex"" (from BSF4ooRexx) on:" dir~runtime)~~say
     ufs~~say("; Created by ""setupFileAssociations.rex"" (from BSF4ooRexx) on:" dir~runtime)~~say

     typeName=dir~entry(regfile".Name")
     mimetype=dir~entry(regfile".MimeType")

      -- add file-extension entries
     do extension over dir~entry( uRegFile".extensions")

        key=hkcr || "\" || extension~upper || "\"
        keyVal=queryWinRegistry(wsh, key)
        if keyVal<>.nil then           -- if set up, don't change it
        do
           ifs~~say("; skipping existing key:" pp(hkcr || "\" || extension~upper)) ~~say   -- add/update key
           iterate
        end

        ifs~say(pp(hkcr || "\" || extension~upper))    -- add/update key
        ifs~say('@=' || enQQ(typeName))
        ifs~say('"Content Type"=' || enQQ(mimetype) )
        ifs~say('"PerceivedType"="text"')

        ufs~say(pp('-' || hkcr || "\" || extension~upper)) -- remove entire key

        ifs~say
        ufs~say
     end


         -- set template file name for ShellNew
     fname=stream("..\template\"dir~entry(regfile".template"), "C", "query exists") /* from current directory */
say "---> fname:" pp(fname)
     if fname<>"" then
     do
        shellNewKey = hkcr || "\."uRegFile"\ShellNew\"
        shellNewVal = queryWinRegistry(wsh, shellNewKey)
        if shellNewVal=.nil then
        do
            -- add mimetype information
           ifs~say(pp(hkcr || "\."uRegFile"\ShellNew"))   -- add/update key
           ifs~say('"FileName"=' || quote4reg(fname) )

           ufs~say(pp("-" ||  hkcr || "\."uRegFile"\ShellNew")) -- remove this particular key
           ufs~say
        end
        else
        do
           ifs~say(";" pp(hkcr || "\."uRegFile"\ShellNew"))    -- add/update key
           ifs~say("; key exists already:" pp(shellNewKey))    -- add/update key
        end
        ifs~say
     end


      -- add file-extension to MIME DataBase
     ifs~say(pp(hkcr || "\MIME\DataBase\Content Type\" || mimetype))  -- add/update key
     ifs~say('"Extension"=' || enQQ("."regFile) )
     ifs~say

     ufs~say(pp('-' || hkcr || "\MIME\DataBase\Content Type\" || mimetype))  -- remove key
     ufs~say


     -- add typeName definitions
     ifs~say
     ifs~say(pp(hkcr || "\" || typename))    -- add/update key
     ufs~say(pp('-' || hkcr || "\" || typename))    -- remove key

     ifs~say('@=' || enQQ(dir~entry(regFile".comment")))  -- description/comment of entry
     ifs~say

            -- define icon
     ifs~say(pp(hkcr || "\" || typename || "\DefaultIcon"))
     fname=stream(dir~path_to_images || "\" || dir~entry(uRegFile".ImageName") || ".ico", "C", "query exists")
-- TODO: check whether we need to enquote the file-name part (if blanks are in the path)
     ifs~~say('@='quote4reg(fname",0"))~~say

            -- define shell
     ifs~say(pp(hkcr || "\" || typename || "\shell"))
     ifs~~say('@="open"')~~say

            -- define shell edit
     ifs~say(pp(hkcr || "\" || typename || "\shell\edit"))
     ifs~~say('@="Edit"')~~say

            -- define shell edit command
     ifs~say(pp(hkcr || "\" || typename || "\shell\edit\command"))
     ifs~~say('@=' || quote4reg('notepad.exe "%1"'))~~say


            -- define shell open
     ifs~say(pp(hkcr || "\" || typename || "\shell\open"))
     ifs~~say('@="Run"')~~say

            -- define shell open command
     ifs~say(pp(hkcr || "\" || typename || "\shell\open\command"))
     ifs~say('@=' || quote4reg(enQQ(dir~full_path_to_rexx) '"%1" %*'))

      -- done
     ifs~close
     ufs~close
  end
  return




/* ================================================================================= */
/* -------------------- Linux part ---------------------- */
/*
   Maybe useful URLs (as of 2010-07-27):

   - Icon Theme Specification
      <http://standards.freedesktop.org/icon-theme-spec/icon-theme-spec-latest.html>
   - Desktop Entry Specification
      <http://standards.freedesktop.org/desktop-entry-spec/desktop-entry-spec-latest.html>
   - Desktop Menu Specification
      <http://standards.freedesktop.org/menu-spec/latest/>
   - XDG Base Directory Specification
      <http://standards.freedesktop.org/basedir-spec/basedir-spec-latest.html>
   - Shared MIME-info Database
      <http://standards.freedesktop.org/shared-mime-info-spec/shared-mime-info-spec-latest.html>
   - Modifying MIME types (GNOME Documentation Library)
      <http://library.gnome.org/admin/system-admin-guide/stable/mimetypes-modifying.html.en>

*/
::routine create4linux
  use arg dir




/* ================================================================================= */
/* -------------------- Miscellaneous ------------------- */
::routine pp
  return "["arg(1)"]"

::routine quote4reg  -- qote a string (e.g. fully qualified path) according to the Windows registry rules
  parse arg str

  newStr=str~changeStr("\", "\\")      -- escape regular backslashes
  newStr=newStr~changeStr('"', '\"')   -- escape double-quotes
  return enQQ(newStr)

::routine enQQ       -- enclose string in double quotes
  parse arg str
  return '"' || str || '"'

::routine queryWinRegistry    -- reads the given key from the registry
  use arg wsh, key
  signal on syntax            -- will be raised as OLE error, if key cannot be read/opened

  return wsh~regRead(key)     -- return value stored with key

syntax:                       -- key could not be read/opened
  return .nil                 -- return .nil

